﻿using System;
using System.IO;
using System.Reflection;
using FreshMvvm;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace FreshMVVMSFDataGrid.ViewModel
{
    public class TemplateViewModelBase : FreshBasePageModel
    {

        #region Property Template

        private StackLayout _template;

        public StackLayout Template
        {
            get
            {
                if (_template != null)
                {
                    RemoveBehaviors(_template);
                }
                _template = CreateTemplate(this);
                return _template;
            }
            set
            {
                _template = value;
            }
        }

        #endregion;

        private void RemoveBehaviors(Xamarin.Forms.View parent)
        {
            if (parent != null)
            {
                parent.Behaviors.Clear();
                Layout<Xamarin.Forms.View> parentView = parent as Layout<Xamarin.Forms.View>;
                ScrollView parentScrollView = parent as ScrollView;
                if (parentView != null)
                {
                    foreach (Xamarin.Forms.View view in parentView.Children)
                    {
                        RemoveBehaviors(view);
                    }
                }
                else if (parentScrollView != null)
                {
                    parentScrollView.Content.Behaviors.Clear();
                    RemoveBehaviors(parentScrollView.Content);
                }
            }
        }

        public StackLayout CreateTemplate(TemplateViewModelBase templateViewModel)
        {
            string xamlTemplate = GetXamlTemplate(templateViewModel);
            StackLayout templateControlFromXaml = CreateTemplateControlFromXaml(xamlTemplate);
            templateControlFromXaml.BindingContext = templateViewModel;
            return templateControlFromXaml;
        }

        private string GetXamlTemplate(TemplateViewModelBase templateViewModel)
        {
            string templateViewName = GetTemplateViewName(templateViewModel);
            string templateXaml = string.Empty;

            Assembly assembly = typeof(TemplateViewModelBase).GetTypeInfo().Assembly;
            Stream stream = assembly.GetManifestResourceStream($"FreshMVVMSFDataGrid.View.Template.{templateViewName}.xaml");
            
            if (stream != null)
            {
                stream.Position = 0;
                using (StreamReader reader = new StreamReader(stream))
                {
                    templateXaml = reader.ReadToEnd();
                }
            }

            return templateXaml;
        }

        private static string GetTemplateViewName(TemplateViewModelBase templateViewModel)
        {
            string templateViewName = templateViewModel.GetType().ToString();
            int lastDot = templateViewName.LastIndexOf(".", StringComparison.Ordinal);
            int modelLength = 5;
            templateViewName = templateViewName.Substring(lastDot + 1, templateViewName.Length - 1 - lastDot - modelLength);
            return templateViewName;
        }

        private StackLayout CreateTemplateControlFromXaml(string templateXaml)
        {
            StackLayout stackLayout = new StackLayout();
            stackLayout.LoadFromXaml(templateXaml);
            return stackLayout;
        }
        
    }
}
